# Example of array indexing
# Copy N locations from A to B
#   read in N
#   copy A[i] to B[i] for i = 0 .. N-1

# idiom to initialize PC with START addr
_:START;

START:
SUB, IN, _0, N;

# addr_A is pointer to src A item
# addr_B is pointer to dst B item

# Modify the copy instruction 
#       SUB, I1, _0, I3
# below to replace the instruction components I1 and I3 with the address of 
#   A[0] in I1
#   B[0] in I3
SUB, addr_A, _0, copy+1;
SUB, addr_B, _0, copy+3;

# The test is at the end of the loop, so
# we start at i = 1, so technically we are
# doing B[i-1] = A[i-1]
# This is the kind of code mangling a compiler does!

SUB, _1, _0, i;
BGE, _0, _0, test;

loop:
    # Move one element from A to B
    copy: SUB, 0, _0, 0;

    # Advance to next elements by incrementing i and the
    # 1st and 3rd instruction fields!
    SUB, i, _m1, i;
    SUB, copy+1, _m1, copy+1;
    SUB, copy+3, _m1, copy+3;
test: BGE, N, i, loop;

done: SUB, _0, _0, PC;

_m1: -1;
_0: 0;
_1: 1;

i: 0;
N: 0;

addr_A: A;
addr_B: B;

A:
10;
11;
12;
13;
14;
15;

B: 
20;
21;
22;
23;
24;
25;
26;
27;
