# Program to show stack-based calling conventions

# Read in n
# Compute the following recursive function fn(n)
#   fn(0) = 1;
#   fn(n) = 2 * fn(n-1);
# that computes fn(n) = 2 ** n

# loc 0 is always PC by hardware
# loc 1 is stack pointer sp by convention
# loc 2 is address of bottom of stack, 
#   grows upward to increasing addrs

# Idiom to initialize PC with START addr
_:START;

sp: stack;
addr_stack: stack;

# Constants, convention is _ddddd or _nddddd.  
# e.g. _12 for 12, _n2 for -2

_0: 0;
_1: 1;
_2: 2;
_3: 3;
_n1: -1;
_n2: -2;
_n3: -3;
_n8: -8;

# We compute m = fn(n) = 2 ** n
n: 0; 
m: 0;

# count the number of calls
n_calls: 0;

# Temp variables, be careful not to use across a fn call
# just like a register
tmp1: 0;
result: 0;

# Always reset the stack pointer when you start
START: SUB, addr_stack, _0, sp;

# A stack frame looks like this:
# [[sp]-2] - function param
# [[sp]-1] - return result
# [[sp]] - return address

# Read n
SUB, IN, _0, n;

# Call fn(n), 
# Allocate space on the stack for param n, result m, return addr 
SUB, sp, _n3, sp;

# Put n into the param position at [sp]-2
# i.e. [ n ] -> [ [sp]-2 ]
SUB, sp, _2, @+7;
SUB, n, _0, 0;

# Put 0 into the result position at [sp]-1 to 0
# i.e. 0 -> [ [sp]-1 ]
SUB, sp, _1, @+7;
SUB, _0, _0, 0;

# Put return address (next instruction after the call) into 
# the return position at [sp].  
SUB, sp, _0, @+7;
SUB, PC, _n8, 0;

# Call the function
BGE, _0, _0, fn;
# Return here

# Fetch result
SUB, sp, _1, @+5;
SUB, 0, _0, m;

# Pop stack frame: param, result, and return address 
SUB, sp, _3, sp;

# Print the result
SUB, m, _0, OUT;

# Print the the number of recursive calls
SUB, n_calls, _0, OUT;

# Halt by setting PC to 0
SUB, PC, PC, PC;

# Body of function fn
# fn(0) = 1;
# fn(n) = 2 * fn(n-1);
fn:

# Count calls
SUB, n_calls, _n1, n_calls;

# Fetch the param n into tmp1, the global copy of n, not
# safe over a function call
SUB, sp, _2, @+5;
SUB, 0, _0, tmp1;

# Don't recurse if n <= 0;
BGE, _0, tmp1, base_case;

# Recursive call, put new param on stack

# Allocate space on the stack for param n, result m, return addr 
SUB, sp, _n3, sp;

# Put n-1 into the param position.
SUB, sp, _2, @+7;
SUB, tmp1, _1, 0;

# Clear result location
SUB, sp, _1, @+7;
SUB, _0, _0, 0;

# Return address
SUB, sp, _0, @+7;
SUB, PC, _n8, 0;

# Call fn.
BGE, _0, _0, fn;
# return here 

# Fetch result
SUB, sp, _1, @+5;
SUB, 0, _0, tmp1;

# Pop stack frame
SUB, sp, _3, sp;

# Multiply result by 2
# result is unsafe over function calls
SUB, _0, tmp1, result;
SUB, result, tmp1, result;
SUB, _0, result, result;

BGE, _0, _0, done;

base_case:
# Result is 1
SUB, _1, _0, result;

# now return to the caller

done:
# Return, stack to be cleaned up by caller who is only 
# one that knows how many things were put on the stack.

# Put returned result into stack, just below return address
SUB, sp, _1, @+7;
SUB, result, _0, 0;

# Get the return address from top of the stack
# and install it into the branch
SUB, sp, _0, @+5;
SUB, 0, _0, @+7;
BGE, _0, _0, 0;

# Stack starts here
stack: 0;
# Advance a bit so that the trace routine shows the stack cells
# with labels
@ = @ + 32
stack_end: 0;
