# 
# Start loading into memory at location 12
#
@ = 12;
# 
# Some simple useful constants
#
Zero: 0;
NegOne: -1;

# Arithmetic on symbols to initialize constants
_far_delta: here - far;

# Instead of naming constants, we can adopt the convention of 
# _x for the constant x, and _nx for the constant -x
_n1: -1;
_0: 0;
_1: 1;
_2: 2;

#
# Some variables
#
A: 10;
B: 0;
X: 42;
Y: -42;
Z: 0;

#
# Copy from location A to location B, i.e. [A] -> [B]
#

SUB, A, Zero, B;

#
# Increment location T by 1, i.e. [T] + 1 -> [T]
#

SUB, T, NegOne, T;

#
# Branch to location Dest: Dest -> [PC]
#

BGE, Zero, Zero, Dest;
# ...
Dest:

#
# Test if location X and location Y have the same values.  I.e.
# if [X] == [Y] then do something, otherwise do something else.
#

BGE, X, Y, ge;
BGE, Zero, Zero, ne;
ge: BGE, Y, X, eq;
BGE, Zero, Zero, ne;
eq: 
# do stuff if equal, branch when done
BGE, Zero, Zero, done;
ne: 
# do stuff if not equal, branch when done
BGE, Zero, Zero, done;
# ...
done:

#
# Compute the min of location X and Y and store in location Z, 
# i.e. min( [X], [Y] ) -> [Z]
#

BGE, Y, X, xmin;
ymin: SUB, Y, _0, Z;
BGE, _0, _0, next;
xmin: SUB, X, _0, Z;
next:
# ...

#
# Using the @ symbol, which has the value of the address of the
# beginning of the instruction, you can eliminate having to 
# invent unique names for intermediate symbols.  Best when 
# using a macro-processor like M4.
#

BGE, Y, X, @+16;
SUB, Y, _0, Z;
BGE, _0, _0, @+8;
SUB, X, _0, Z;

# 
# Relative branch 2 instructions after "here" by modifying PC.
# During instruction execution PC is not changed until instruction
# complete, which is why [_far_delta] is -12 and not -8.
#

SUB, _0, _0, OUT;
here: SUB, PC, _far_delta, PC;

near: SUB, _1, _0, OUT;
SUB, PC, PC, PC;

far: SUB, _2, _0, OUT;
SUB, PC, PC, PC;

# 
# Indexed load from element L[i] of an array: [L+i] -> [V]
#

SUB, _0, i, T;
SUB, L_addr, T, load+1;
load: SUB, 0, _0, V;
# ...
# index
i: 0; 
# temp 
T: 0; 
# value to work with
V: 0; 
# pointer to array L
L_addr: L; 
# number of elements in L
L_len: L_end - L; 
# start of array L
L:
@ = @ + 10; # 10 un-initialized locations for L
# 1 past end of array L, so L_end - L is length of array
L_end: ; 
